@@ -23,8 +23,6 @@
 # this script exports dbc-files from a canmatrix-object
 # dbc-files are the can-matrix-definitions of the CANoe (Vector Informatic)
 
-from __future__ import absolute_import, division, print_function
-
 import collections
 import copy
 import decimal
@@ -225,6 +223,8 @@ def dump(in_db, f, **options):
         # remove "-" from frame names
         if compatibility:
             frame.name = re.sub("[^A-Za-z0-9]", whitespace_replacement, frame.name)
+            if frame.name[0].isdigit():
+                frame.name = "_" + frame.name
 
         duplicate_signal_totals = collections.Counter(normalized_names.values())
         duplicate_signal_counter = collections.Counter()  # type: typing.Counter[str]
@@ -237,14 +237,14 @@ def dump(in_db, f, **options):
         for signal in frame.signals:
             if signal.cycle_time != 0:
                 signal.add_attribute("GenSigCycleTime", signal.cycle_time)
+            if signal.initial_value != 0 and "GenSigStartValue" not in db.signal_defines:
+                db.add_signal_defines("GenSigStartValue", 'FLOAT 0 100000000000')
+                
             if "GenSigStartValue" in db.signal_defines:
                 if signal.phys2raw(None) != 0:
-                    if db.signal_defines["GenSigStartValue"].defaultValue is not None and \
-                            float(signal.initial_value) != float(db.signal_defines["GenSigStartValue"].defaultValue):
-                        signal.add_attribute("GenSigStartValue", signal.phys2raw(float(db.signal_defines["GenSigStartValue"].defaultValue)))
-                    elif db.signal_defines["GenSigStartValue"].defaultValue is None:
+                    if db.signal_defines["GenSigStartValue"].defaultValue is None:
                         signal.add_attribute("GenSigStartValue", signal.phys2raw(None))
-
+                        
             name = normalized_names[signal]
             if compatibility:
                 name = re.sub("[^A-Za-z0-9]", whitespace_replacement, name)
@@ -461,7 +461,7 @@ def dump(in_db, f, **options):
         if frame.is_complex_multiplexed:
             for signal in frame.signals:
                 if signal.muxer_for_signal is not None:
-                    f.write(("SG_MUL_VAL_ %d %s %s " % (frame.arbitration_id.to_compound_integer(), signal.name, signal.muxer_for_signal)).encode(dbc_export_encoding, ignore_encoding_errors))
+                    f.write(("SG_MUL_VAL_ %d %s %s " % (frame.arbitration_id.to_compound_integer(), output_names[frame][signal], signal.muxer_for_signal)).encode(dbc_export_encoding, ignore_encoding_errors))
                     f.write((", ".join(["%d-%d" % (a, b) for a, b in signal.mux_val_grp])).encode(dbc_export_encoding, ignore_encoding_errors))
 
                     f.write(";\n".encode(dbc_export_encoding, ignore_encoding_errors))
@@ -558,7 +558,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                 if decoded.strip().endswith(r'"'):
                     decoded += r" Vector__XXX"
                     original_line += b" Vector__XXX"
-                pattern = r"^SG_ +(\w+) *: *(\d+)\|(\d+)@(\d+)([\+|\-]) *\(([0-9.+\-eE]+), *([0-9.+\-eE]+)\) *\[([0-9.+\-eE]+)\|([0-9.+\-eE]+)\] +\"(.*)\" +(.*)"
+                pattern = r"^SG_ +(\S+) *: *(\d+)\|(\d+)@(\d+)([\+|\-]) *\(([0-9.+\-eE]+), *([0-9.+\-eE]+)\) *\[([0-9.+\-eE]+)\|([0-9.+\-eE]+)\] +\"(.*)\" +(.*)"
                 regexp = re.compile(pattern)
                 temp = regexp.match(decoded)
                 regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
@@ -599,9 +599,11 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                     multiplex = temp.group(2)  # type: str
 
                     is_complex_multiplexed = False
+                    is_multiplexer = False
 
                     if multiplex == 'M':
                         multiplex = 'Multiplexor'
+                        is_multiplexer = True
                     elif multiplex.endswith('M'):
                         is_complex_multiplexed = True
                         multiplex = multiplex[:-1]
@@ -632,7 +634,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                         **extras
                     )
 
-                    if is_complex_multiplexed:
+                    if is_complex_multiplexed or is_multiplexer:
                         temp_signal.is_multiplexer = True
                         temp_signal.multiplex = 'Multiplexor'
 
@@ -651,7 +653,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                 for ecu_name in temp.group(2).split(','):
                     frame.add_transmitter(ecu_name)
             elif decoded.startswith("CM_ SG_ "):
-                pattern = r"^CM_ +SG_ +(\w+) +(\w+) +\"(.*)\" *;"
+                pattern = r"^CM_ +SG_ +(\S+) +(\S+) +\"(.*)\" *;"
                 regexp = re.compile(pattern)
                 regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                 temp = regexp.match(decoded)
@@ -668,7 +670,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                                 "Error decoding line: %d (%s)" %
                                 (i, line))
                 else:
-                    pattern = r"^CM_ +SG_ +(\w+) +(\w+) +\"(.*)"
+                    pattern = r"^CM_ +SG_ +(\S+) +(\S+) +\"(.*)"
                     regexp = re.compile(pattern)
                     regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                     temp = regexp.match(decoded)
@@ -686,7 +688,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                         follow_up = _FollowUps.SIGNAL_COMMENT
 
             elif decoded.startswith("CM_ BO_ "):
-                pattern = r"^CM_ +BO_ +(\w+) +\"(.*)\" *;"
+                pattern = r"^CM_ +BO_ +(\S+) +\"(.*)\" *;"
                 regexp = re.compile(pattern)
                 regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                 temp = regexp.match(decoded)
@@ -702,7 +704,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                                 "Error decoding line: %d (%s)" %
                                 (i, line))
                 else:
-                    pattern = r"^CM_ +BO_ +(\w+) +\"(.*)"
+                    pattern = r"^CM_ +BO_ +(\S+) +\"(.*)"
                     regexp = re.compile(pattern)
                     regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                     temp = regexp.match(decoded)
@@ -718,7 +720,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                                 (i, line))
                         follow_up = _FollowUps.FRAME_COMMENT
             elif decoded.startswith("CM_ BU_ "):
-                pattern = r"^CM_ +BU_ +(\w+) +\"(.*)\" *;"
+                pattern = r"^CM_ +BU_ +(\S+) +\"(.*)\" *;"
                 regexp = re.compile(pattern)
                 regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                 temp = regexp.match(decoded)
@@ -732,7 +734,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                         except:
                             logger.error("Error decoding line: %d (%s)" % (i, line))
                 else:
-                    pattern = r"^CM_ +BU_ +(\w+) +\"(.*)"
+                    pattern = r"^CM_ +BU_ +(\S+) +\"(.*)"
                     regexp = re.compile(pattern)
                     regexp_raw = re.compile(pattern.encode(dbc_import_encoding))
                     temp = regexp.match(decoded)
@@ -760,14 +762,14 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                             db.ecus.append(canmatrix.Ecu(ele))
 
             elif decoded.startswith("VAL_ "):
-                regexp = re.compile(r"^VAL_ +(\w+) +(\w+) +(.*) *;")
+                regexp = re.compile(r"^VAL_ +(\d+)? *(\S+) +(.*) *;")
                 temp = regexp.match(decoded)
                 if temp:
                     frame_id = temp.group(1)
                     signal_name = temp.group(2)
                     temp_list = list(canmatrix.utils.escape_aware_split(temp.group(3), '"'))
 
-                    if frame_id.isnumeric():  # value for Frame
+                    if frame_id:  # value for Frame
                         try:
                             frame = get_frame_by_id(canmatrix.ArbitrationId.from_compound_integer(int(frame_id)))
                             sg = frame.signal_by_name(signal_name)
@@ -778,11 +780,18 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                                     sg.add_values(temp_list[i * 2], val)
                         except:
                             logger.error("Error with Line: " + str(temp_list))
-                    else:
-                        logger.info("Warning: environment variables currently not supported")
+                    else:                        
+                        try:
+                            values = db.env_vars[signal_name]['values']
+                            for i in range(math.floor(len(temp_list) / 2)):
+                                val = temp_list[i * 2 + 1]
+                                val = val.replace('\\"', '"')
+                                values[temp_list[i * 2].strip()]=val
+                        except:
+                            logger.error("Error with Line: " + str(temp_list))
 
             elif decoded.startswith("VAL_TABLE_ "):
-                regexp = re.compile(r"^VAL_TABLE_ +(\w+) +(.*) *;")
+                regexp = re.compile(r"^VAL_TABLE_ +(\S+) +(.*) *;")
                 temp = regexp.match(decoded)
                 if temp:
                     table_name = temp.group(1)
@@ -838,18 +847,18 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                     get_frame_by_id(canmatrix.ArbitrationId.from_compound_integer(int(temp.group(2)))).add_attribute(
                         temp.group(1), temp.group(3))
                 elif tempba.group(1).strip().startswith("SG_ "):
-                    regexp = re.compile(r"^BA_ +\"(.+?)\" +SG_ +(\d+) +(\w+) +(.+) *; *")
+                    regexp = re.compile(r"^BA_ +\"(.+?)\" +SG_ +(\d+) +(\S+) +(.+) *; *")
                     temp = regexp.match(decoded)
                     if temp is not None:
                         get_frame_by_id(canmatrix.ArbitrationId.from_compound_integer(int(temp.group(2)))).signal_by_name(
                             temp.group(3)).add_attribute(temp.group(1), temp.group(4))
                 elif tempba.group(1).strip().startswith("EV_ "):
-                    regexp = re.compile(r"^BA_ +\"(.+?)\" +EV_ +(\w+) +(.*) *; *")
+                    regexp = re.compile(r"^BA_ +\"(.+?)\" +EV_ +(\S+) +(.*) *; *")
                     temp = regexp.match(decoded)
                     if temp is not None:
                         db.add_env_attribute(temp.group(2), temp.group(1), temp.group(3))
                 elif tempba.group(1).strip().startswith("BU_ "):
-                    regexp = re.compile(r"^BA_ +\"(.*?)\" +BU_ +(\w+) +(.+) *; *")
+                    regexp = re.compile(r"^BA_ +\"(.*?)\" +BU_ +(\S+) +(.+) *; *")
                     temp = regexp.match(decoded)
                     db.ecu_by_name(
                         temp.group(2)).add_attribute(
@@ -857,13 +866,13 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                         temp.group(3))
                 else:
                     regexp = re.compile(
-                        r"^BA_ +\"([A-Za-z0-9\-_]+)\" +([\"\w\-\.]+) *; *")
+                        r"^BA_ +\"([A-Za-z0-9\-_]+)\" +([\"\S\-\.]+) *; *")
                     temp = regexp.match(decoded)
                     if temp:
                         db.add_attribute(temp.group(1), temp.group(2))
 
             elif decoded.startswith("SIG_GROUP_ "):
-                regexp = re.compile(r"^SIG_GROUP_ +(\w+) +(\w+) +(\w+) +\:(.*) *; *")
+                regexp = re.compile(r"^SIG_GROUP_ +(\S+) +(\S+) +(\S+) +\:(.*) *; *")
                 temp = regexp.match(decoded)
                 frame = get_frame_by_id(canmatrix.ArbitrationId.from_compound_integer(int(temp.group(1))))
                 if frame is not None:
@@ -871,7 +880,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                     frame.add_signal_group(temp.group(2), temp.group(3), signal_array)  # todo wrong annotation in canmatrix? Id is a string?
 
             elif decoded.startswith("SIG_VALTYPE_ "):
-                regexp = re.compile(r"^SIG_VALTYPE_ +(\w+) +(\w+)\s*\:(.*) *; *")
+                regexp = re.compile(r"^SIG_VALTYPE_ +(\S+) +(\S+)\s*\:(.*) *; *")
                 temp = regexp.match(decoded)
                 frame = get_frame_by_id(canmatrix.ArbitrationId.from_compound_integer(int(temp.group(1))))
                 if frame:
@@ -889,7 +898,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                     db.add_define_default(temp.group(1),
                                           temp_raw.group(2).decode(dbc_import_encoding))
             elif decoded.startswith("SG_MUL_VAL_ "):
-                pattern = r"^SG_MUL_VAL_ +([0-9]+) +([\w\-]+) +([\w\-]+) +(.*) *; *"
+                pattern = r"^SG_MUL_VAL_ +([0-9]+) +([\S\-]+) +([\S\-]+) +(.*) *; *"
                 regexp = re.compile(pattern)
                 temp = regexp.match(decoded)
                 if temp:
@@ -908,7 +917,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                             mux_val_max_number = int(mux_val_max)
                             signal.mux_val_grp.append([mux_val_min_number, mux_val_max_number])
             elif decoded.startswith("EV_ "):
-                pattern = r"^EV_ +([\w\-\_]+?) *\: +([0-9]+) +\[([0-9.+\-eE]+)\|([0-9.+\-eE]+)\] +\"(.*?)\" +([0-9.+\-eE]+) +([0-9.+\-eE]+) +([\w\-]+?) +(.*); *"
+                pattern = r"^EV_ +([\S\-\_]+?) *\: +([0-9]+) +\[([0-9.+\-eE]+)\|([0-9.+\-eE]+)\] +\"(.*?)\" +([0-9.+\-eE]+) +([0-9.+\-eE]+) +([\S\-]+?) +(.*); *"
                 regexp = re.compile(pattern)
                 temp = regexp.match(decoded)
 
@@ -923,7 +932,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
                 access_nodes = temp.group(9).split(",")
                 db.add_env_var(var_name, {"varType": var_type, "min": min_value, "max": max_value, "unit": unit,
                                           "initialValue": initial_value, "evId": ev_id, "accessType": access_type,
-                                          "accessNodes": access_nodes})
+                                          "accessNodes": access_nodes, "values": {}})
 
         # else:
         except:
@@ -989,6 +998,7 @@ def load(f, **options):  # type: (typing.IO, **typing.Any) -> canmatrix.CanMatri
 
     db.enum_attribs_to_values()
     for frame in db.frames:
+        frame.multiplex_signals()
         if "_FD" in frame.attributes.get("VFrameFormat", ""):
             frame.is_fd = True
         if "J1939PG" in frame.attributes.get("VFrameFormat", ""):
