@@ -25,8 +25,7 @@
 # only (fibex: Field Bus Exchange Format //
 # https://de.wikipedia.org/wiki/Field_Bus_Exchange_Format)
 
-from __future__ import absolute_import, division, print_function
-
+import os
 import typing
 from builtins import *
 import lxml.etree
@@ -78,6 +77,85 @@ def create_sub_element_ho(parent, element_name, element_text=None):
     return new
 
 
+def create_signal_instance(parent, signal, signal_id):
+    # type: (_Element, Signal, str) -> _Element
+    signal_instance = create_sub_element_fx(parent, "SIGNAL-INSTANCE")
+    signal_instance.set("ID", "PDUINST_" + signal_id)
+    create_sub_element_fx(signal_instance, "BIT-POSITION", str(signal.get_startbit(bit_numbering=1)))
+    create_sub_element_fx(signal_instance, "IS-HIGH-LOW-BYTE-ORDER", "false" if signal.is_little_endian else "true")
+    return signal_instance
+
+
+def create_signal_ref(parent, signal_ref_id):
+    # type: (_Element, str) -> _Element
+    signal_ref = create_sub_element_fx(parent, "SIGNAL-REF")
+    signal_ref.set("ID-REF", "SIG_" + signal_ref_id)
+    return signal_ref
+
+
+def bits_to_byte_str(bits, name):
+    b = int(bits)
+    if b % 8 == 0:
+        return str(int(b/8))
+    else:
+        ret = int(b/8) + 1
+        #if name is not None:
+        #    print(f"Warning: {name} not byte aligned! {b} bits -> {ret} bytes")
+        return str(ret)
+
+
+def create_signal_id(frame, signal):
+    # type: (Frame, Signal) -> str
+    return f"{frame.name}.{signal.name}_{str(signal.mux_val)}" if signal.mux_val else f"{frame.name}.{signal.name}"
+
+
+def get_multiplexing_parts_infos(signals, frame_name, start_pos=-1, end_pos=-1, seg_big_endian=None):
+    # type: (list[Signal], str, int, int, bool) -> (int, int, bool)
+
+    for signal in signals:
+        if start_pos == -1 or signal.start_bit < start_pos:
+            start_pos = signal.start_bit
+
+        tmp_end_pos = signal.start_bit + signal.size
+        if end_pos == -1 or tmp_end_pos > end_pos:
+            end_pos = tmp_end_pos
+
+        if seg_big_endian is not None and seg_big_endian == signal.is_little_endian:
+            mux_info = f" with mux_val {signal.mux_val}" if signal.mux_val else ""
+            print(f"Warning: Inconsistent Byte Order for frame {frame_name} {signal.name}{mux_info}")
+        seg_big_endian = not signal.is_little_endian
+
+    return start_pos, end_pos, seg_big_endian
+
+def get_base_data_type(signal):
+    # type: (Signal) -> str
+    if signal.is_float:
+        if (signal.size<=32):
+            return "A_FLOAT32"
+        else:
+            return "A_FLOAT64"
+    
+    if signal.size > 0 and signal.size <= 8:
+        if signal.is_signed:
+            return "A_INT8"
+        elif not signal.is_signed:
+            return "A_UINT8"
+        
+    elif signal.size > 8 and signal.size <= 16:
+            if signal.is_signed:
+                return "A_INT16"
+            elif not signal.is_signed:
+                return "A_UINT16"
+    elif signal.size > 16 and signal.size <= 32:
+            if signal.is_signed:
+                return "A_INT32"
+            elif not signal.is_signed:
+                return "A_UINT32"
+    elif signal.size > 32 and signal.size <= 64:
+            if signal.is_signed:
+                return "A_INT64"
+            elif not signal.is_signed:
+                return "A_UINT64"
 class Fe:
     def __init__(self, filename):
         self.tree = lxml.etree.parse(filename)
@@ -200,10 +278,18 @@ def get_signals_for_pdu(fe, pdu, overall_startbit = 0):
 
         signal_name = fe.sn(signal)
         coding = fe.selector(signal, ">CODING-REF")[0]
+        is_signed = False
+        try:
+            basedatatype = fe.selector(coding, "/!CODED-TYPE")[0].attrib[fe.ans + "BASE-DATA-TYPE"]
+            if "UINT" in basedatatype:
+                is_signed = False
+            elif "INT" in basedatatype:
+                is_signed = True
+        except:
+            pass
         bit_length = int(fe.selector(coding, "/!BIT-LENGTH")[0].text)
         compu_methods = fe.selector(coding, "/!COMPU-METHOD")
-        sig = canmatrix.Signal(name=signal_name)
-
+        sig = canmatrix.Signal(name=signal_name, is_signed=is_signed)
         for compu_method in compu_methods:
             category = fe.selector(compu_method, "/!CATEGORY")
             if len(category) > 0 and category[0].text == "LINEAR":
@@ -337,6 +423,26 @@ def dump(db, f, **options):
         '{{{pre}}}schemaLocation'.format(
             pre=xsi)] = 'http://www.asam.net/xml/fbx ..\\..\\xml_schema\\fibex.xsd http://www.asam.net/xml/fbx/can  ..\\..\\xml_schema\\fibex4can.xsd'
 
+    #
+    # Make sure that we can even write to FIBEX
+    #
+
+    # make frame names unique by adding suffix, if needed
+    frame_names = dict()
+    for frame in db.frames:
+        if frame.name in frame_names:
+            # conflict resolution
+            tmp = 2
+            while f"{frame.name}_{str(tmp)}" in frame_names:
+                tmp += 1
+
+            new_name = f"{frame.name}_{str(tmp)}"
+            print(f"Warning: Changing frame name due to conflict: {frame.name} -> {new_name}")
+            frame.name = new_name
+
+        frame_names[frame.name] = frame
+
+
     #
     # PROJECT
     #
@@ -355,7 +461,9 @@ def dump(db, f, **options):
     clusters = create_sub_element_fx(elements, "CLUSTERS")
     cluster = lxml.etree.SubElement(clusters, ns_fx + "CLUSTER")
     cluster.set('ID', 'canCluster1')
-    create_short_name_desc(cluster, "clusterShort", "clusterDesc")
+    # add the file name as a suffix in the cluster name
+    cluster_name = f"cluster_{os.path.basename(f.name).split('.')[0]}"
+    create_short_name_desc(cluster, cluster_name, "clusterDesc")
     create_sub_element_fx(cluster, "SPEED", "500")
     create_sub_element_fx(cluster, "IS-HIGH-LOW-BIT-ORDER", "true")
     create_sub_element_fx(cluster, "BIT-COUNTING-POLICY", "MONOTONE")
@@ -400,6 +508,9 @@ def dump(db, f, **options):
         create_sub_element_fx(identifier, "IDENTIFIER-VALUE", str(frame.arbitration_id.id))
         frame_ref = create_sub_element_fx(frame_triggering, "FRAME-REF")
         frame_ref.set("ID-REF", "FRAME_" + frame.name)
+        if (frame.is_fd):
+            create_sub_element_fx(frame_triggering, "CAN-FRAME-TX-BEHAVIOR","CAN-FD")
+            create_sub_element_fx(frame_triggering, "CAN-FRAME-RX-BEHAVIOR","CAN-FD")
 
     #
     # ECUS
@@ -437,7 +548,6 @@ def dump(db, f, **options):
                 included_pdu.set('ID', 'input_included_pdu_' + frame.name)
                 pdu_triggering_ref = create_sub_element_fx(included_pdu, "PDU-TRIGGERING-REF")
                 pdu_triggering_ref.set("ID-REF", "PDU_" + frame.name)
-
                 
         outputs = create_sub_element_fx(connector, "OUTPUTS")
         for frame in db.frames:
@@ -451,9 +561,8 @@ def dump(db, f, **options):
                 included_pdu.set('ID', 'output_included_pdu_' + frame.name)
                 pdu_triggering_ref = create_sub_element_fx(included_pdu, "PDU-TRIGGERING-REF")
                 pdu_triggering_ref.set("ID-REF", "PDU_" + frame.name)
-                
-        
-        # ignore CONTROLERS/CONTROLER
+
+        # ignore CONTROLLERS/CONTROLLER
 
     #
     # PDUS
@@ -465,25 +574,130 @@ def dump(db, f, **options):
         create_short_name_desc(pdu, "PDU_" + frame.name, frame.comment)
         create_sub_element_fx(pdu, "BYTE-LENGTH", str(frame.size))  # DLC
         create_sub_element_fx(pdu, "PDU-TYPE", "APPLICATION")
-        signal_instances = create_sub_element_fx(pdu, "SIGNAL-INSTANCES")
-        for signal in frame.signals:
-            signal_id = frame.name + "." + signal.name
-            signal_instance = create_sub_element_fx(
-                signal_instances, "SIGNAL-INSTANCE")
-            signal_instance.set("ID", "PDUINST_" + signal_id)
-            # startBit: TODO - find out correct BYTEORDER ...
-            create_sub_element_fx(signal_instance, "BIT-POSITION",
-                                  str(signal.start_bit))
-            if signal.is_little_endian:
-                create_sub_element_fx(
-                    signal_instance,
-                    "IS-HIGH-LOW-BYTE-ORDER",
-                    "false")  # true:big endian; false:little endian
-            else:
-                create_sub_element_fx(
-                    signal_instance, "IS-HIGH-LOW-BYTE-ORDER", "true")
-            signal_ref = create_sub_element_fx(signal_instance, "SIGNAL-REF")
-            signal_ref.set("ID-REF", "SIG_" + signal_id)
+
+        if frame.is_multiplexed:
+            mux = create_sub_element_fx(pdu, "MULTIPLEXER")
+
+            signals_static = []
+            signals_dynamic = {}
+            signals_switch = []
+
+            for signal in frame.signals:
+                if signal.is_multiplexer:
+                    signals_switch.append(signal)
+                elif signal.multiplex is not None or signal.mux_val is not None:
+                    if signal.multiplex != signal.mux_val:
+                        print(f"Warning: Signal {signal.name} mux_val {signal.mux_val} != multiplex {signal.multiplex}")
+                    sigs = signals_dynamic.setdefault(signal.mux_val, [])
+                    sigs.append(signal)
+                else:
+                    signals_static.append(signal)
+
+            if len(signals_switch) != 1 or len(signals_dynamic) == 0:
+                print(f"Warning: Frame {frame.name} has an invalid multiplexer or dyn Part! "
+                      f"Switch Signals: {len(signals_switch)} "
+                      f"Dyn Signals: {len(signals_dynamic)} "
+                      f"Static Signals: {len(signals_static)}")
+
+            # SWITCH
+            mux_switch = create_sub_element_fx(mux, "SWITCH")
+            mux_switch.set("ID", "PDU_" + frame.name + "_SWITCH")
+            create_sub_element_ho(mux_switch, "SHORT-NAME", signals_switch[0].name)
+            create_sub_element_fx(mux_switch, "BIT-POSITION", str(signals_switch[0].start_bit))
+            create_sub_element_fx(mux_switch, "IS-HIGH-LOW-BYTE-ORDER",
+                                  "false" if signals_switch[0].is_little_endian else "true")
+            create_sub_element_ho(mux_switch, "BIT-LENGTH", str(signals_switch[0].size))
+
+            # DYNAMIC PART
+            start_pos = -1
+            end_pos = -1
+            seg_be = None
+
+            for sigs in signals_dynamic.values():
+                start_pos, end_pos, seg_be = get_multiplexing_parts_infos(sigs,
+                                                                          frame.name,
+                                                                          start_pos=start_pos,
+                                                                          end_pos=end_pos,
+                                                                          seg_big_endian=seg_be)
+
+            if seg_be is None:
+                seg_be = False
+
+            if start_pos >= 0:
+                mux_dynpart = create_sub_element_fx(mux, "DYNAMIC-PART")
+                mux_dynpart.set("ID", "PDU_" + frame.name + "_DYN_PART")
+                create_sub_element_ho(mux_dynpart, "SHORT-NAME", signal.name)
+
+                seg_positions = create_sub_element_fx(mux_dynpart, "SEGMENT-POSITIONS")
+                seg_position = create_sub_element_fx(seg_positions, "SEGMENT-POSITION")
+                create_sub_element_fx(seg_position, "BIT-POSITION", str(start_pos))
+                create_sub_element_fx(seg_position, "IS-HIGH-LOW-BYTE-ORDER", "true" if seg_be else "false")
+                create_sub_element_ho(seg_position, "BIT-LENGTH", str(end_pos - start_pos))
+
+                sw_pdu_instances = create_sub_element_fx(mux_dynpart, "SWITCHED-PDU-INSTANCES")
+
+                for mux_key in sorted(signals_dynamic.keys()):
+                    # create PDU for these signals
+                    mux_key_pdu = create_sub_element_fx(pdus, "PDU")
+                    mux_key_pdu_id = f"PDU_{frame.name}_DYNPART_{str(mux_key)}"
+                    mux_key_pdu.set("ID", mux_key_pdu_id)
+                    create_short_name_desc(mux_key_pdu, mux_key_pdu_id, "")
+                    create_sub_element_fx(mux_key_pdu, "BYTE-LENGTH",
+                                          bits_to_byte_str(end_pos - start_pos, mux_key_pdu_id))
+                    create_sub_element_fx(mux_key_pdu, "PDU-TYPE", "APPLICATION")
+
+                    signal_instances = create_sub_element_fx(mux_key_pdu, "SIGNAL-INSTANCES")
+                    for signal in signals_dynamic[mux_key]:
+                        signal_id = create_signal_id(frame, signal)
+                        signal_instance = create_signal_instance(signal_instances, signal, signal_id)
+                        create_signal_ref(signal_instance, signal_id)
+
+                    # create SWITCHED-PDU-INSTANCE that refs mux_key_pdu_id
+                    switched_pdu_inst = create_sub_element_fx(sw_pdu_instances, "SWITCHED-PDU-INSTANCE")
+                    switched_pdu_inst.set("ID", mux_key_pdu_id + "_SWITCHED_PDU_INSTANCE")
+                    pdu_ref = create_sub_element_fx(switched_pdu_inst, "PDU-REF")
+                    pdu_ref.set("ID-REF", mux_key_pdu_id)
+                    create_sub_element_fx(switched_pdu_inst, "SWITCH-CODE", str(mux_key))
+
+            # STATIC PART
+            start_pos, end_pos, seg_be = get_multiplexing_parts_infos(signals_static, frame.name)
+
+            if start_pos != -1 and end_pos != -1:
+                static_part = create_sub_element_fx(mux, "STATIC-PART")
+                static_part.set("ID", "PDU_" + frame.name + "_STATIC_PART")
+                create_sub_element_ho(static_part, "SHORT-NAME", signal.name)
+                seg_positions = create_sub_element_fx(static_part, "SEGMENT-POSITIONS")
+                seg_position = create_sub_element_fx(seg_positions, "SEGMENT-POSITION")
+                create_sub_element_fx(seg_position, "BIT-POSITION", str(start_pos))
+                create_sub_element_fx(seg_position, "IS-HIGH-LOW-BYTE-ORDER", "true" if seg_be else "false")
+                create_sub_element_ho(seg_position, "BIT-LENGTH", str(end_pos - start_pos))
+
+                # create static PDU
+                static_pdu = create_sub_element_fx(pdus, "PDU")
+                static_pdu_key = f"PDU_{frame.name}_STATIC_PART_PDU"
+                static_pdu.set("ID", static_pdu_key)
+                create_short_name_desc(static_pdu, static_pdu_key, "")
+                create_sub_element_fx(static_pdu, "BYTE-LENGTH", bits_to_byte_str(end_pos - start_pos, frame.name))
+                create_sub_element_fx(static_pdu, "PDU-TYPE", "APPLICATION")
+                signal_instances = create_sub_element_fx(static_pdu, "SIGNAL-INSTANCES")
+                for signal in signals_static:
+                    signal_id = create_signal_id(frame, signal)
+                    signal_instance = create_signal_instance(signal_instances, signal, signal_id)
+                    create_signal_ref(signal_instance, signal_id)
+
+                # create STATIC-PDU-INSTANCE that refs static_pdu_key
+                static_pdu_inst = create_sub_element_fx(static_part, "STATIC-PDU-INSTANCE")
+                static_pdu_inst.set("ID", static_pdu_key + "_SWITCHED-PDU-INSTANCE")
+                pdu_ref = create_sub_element_fx(static_pdu_inst, "PDU-REF")
+                pdu_ref.set("ID-REF", static_pdu_key)
+
+        else:
+            # PDU without Multiplexing
+            signal_instances = create_sub_element_fx(pdu, "SIGNAL-INSTANCES")
+            for signal in frame.signals:
+                signal_id = create_signal_id(frame, signal)
+                signal_instance = create_signal_instance(signal_instances, signal, signal_id)
+                create_signal_ref(signal_instance, signal_id)
 
     # FRAMES
     #
@@ -513,26 +727,24 @@ def dump(db, f, **options):
         input_ports = create_sub_element_fx(function, "INPUT-PORTS")
         for frame in db.frames:
             for signal in frame.signals:
-                signal_id = frame.name + "." + signal.name
+                signal_id = create_signal_id(frame, signal)
                 if bu.name in signal.receivers:
                     input_port = create_sub_element_fx(input_ports, "INPUT-PORT")
                     input_port.set("ID", "INP_" + signal_id)
                     desc = lxml.etree.SubElement(input_port, ns_ho + "DESC")
                     desc.text = signal.comment
-                    signal_ref = create_sub_element_fx(input_port, "SIGNAL-REF")
-                    signal_ref.set("ID-REF", "SIG_" + signal_id)
+                    create_signal_ref(input_port, signal_id)
 
         output_ports = create_sub_element_fx(function, "OUTPUT-PORTS")
         for frame in db.frames:
             if bu.name in frame.transmitters:
                 for signal in frame.signals:
-                    signal_id = frame.name + "." + signal.name
+                    signal_id = create_signal_id(frame, signal)
                     output_port = create_sub_element_fx(output_ports, "OUTPUT-PORT")
                     output_port.set("ID", "OUTP_" + signal_id)
                     desc = lxml.etree.SubElement(output_port, ns_ho + "DESC")
                     desc.text = "signalcomment"
-                    signal_ref = create_sub_element_fx(output_port, "SIGNAL-REF")
-                    signal_ref.set("ID-REF", "SIG_" + frame.name + "_" + signal_id)
+                    create_signal_ref(output_port, frame.name + "_" + signal_id)
 
     #
     # SIGNALS
@@ -540,10 +752,12 @@ def dump(db, f, **options):
     signals = create_sub_element_fx(elements, "SIGNALS")
     for frame in db.frames:
         for signal in frame.signals:
-            signal_id = frame.name + "." + signal.name
+            signal_id = create_signal_id(frame, signal)
             signal_element = create_sub_element_fx(signals, "SIGNAL")
             signal_element.set("ID", "SIG_" + signal_id)
             create_short_name_desc(signal_element, signal.name, signal.comment)
+            default_value = str(signal.phys2raw(signal.initial_value))
+            create_sub_element_fx(signal_element, "DEFAULT-VALUE", default_value)
             coding_ref = create_sub_element_fx(signal_element, "CODING-REF")
             coding_ref.set("ID-REF", "CODING_" + signal_id)
 
@@ -554,7 +768,7 @@ def dump(db, f, **options):
     unit_spec = create_sub_element_ho(proc_info, "UNIT-SPEC")
     for frame in db.frames:
         for signal in frame.signals:
-            signal_id = frame.name + "." + signal.name
+            signal_id = create_signal_id(frame, signal)
             unit = create_sub_element_ho(unit_spec, "UNIT")
             unit.set("ID", "UNIT_" + signal_id)
             create_sub_element_ho(unit, "SHORT-NAME", signal.name)
@@ -563,7 +777,7 @@ def dump(db, f, **options):
     codings = create_sub_element_fx(proc_info, "CODINGS")
     for frame in db.frames:
         for signal in frame.signals:
-            signal_id = frame.name + "." + signal.name
+            signal_id = create_signal_id(frame, signal)
             coding = create_sub_element_fx(codings, "CODING")
             coding.set("ID", "CODING_" + signal_id)
             create_short_name_desc(
@@ -574,6 +788,10 @@ def dump(db, f, **options):
                 signal_id)
                 
             coded = create_sub_element_ho(coding, "CODED-TYPE")
+            base_data_type = get_base_data_type(signal)
+            if base_data_type is not None:
+                coded.set(ns_ho + "BASE-DATA-TYPE", base_data_type)
+                coded.set("CATEGORY", "STANDARD-LENGTH-TYPE")
             create_sub_element_ho(coded, "BIT-LENGTH", str(signal.size))
             
             compu_methods = create_sub_element_ho(coding, "COMPU-METHODS")
