@@ -23,16 +23,17 @@
 # this script exports xls-files from a canmatrix-object
 # xls-files are the can-matrix-definitions displayed in Excel
 
-from __future__ import absolute_import, division, print_function
-
 import logging
 import typing
 from builtins import *
 
-import xlsxwriter
+import openpyxl
+import openpyxl.utils
 
 import canmatrix
 import canmatrix.formats.xls_common
+from openpyxl.worksheet.dimensions import ColumnDimension
+from openpyxl.styles import NamedStyle, Font, Alignment, PatternFill, Border, Side
 
 logger = logging.getLogger(__name__)
 
@@ -80,22 +81,28 @@ def write_ecu_matrix(ecu_list, signal, frame, worksheet, row, col, first_frame):
             loc_style_sender = sender_green
         # write "s" "r" "r/s" if signal is sent, received or send and received by ECU
         if signal is not None and ecu in signal.receivers and ecu in frame.transmitters:
-            worksheet.write(row, col, "r/s", loc_style_sender)
+            worksheet.cell(row=row+1, column=col+1).value = "r/s"
+            worksheet.cell(row=row+1, column=col+1).style = loc_style_sender
         elif signal is not None and ecu in signal.receivers:
-            worksheet.write(row, col, "r", loc_style)
+            worksheet.cell(row=row+1, column=col+1).value = "r"
+            worksheet.cell(row=row+1, column=col+1).style = loc_style
         elif ecu in frame.transmitters:
-            worksheet.write(row, col, "s", loc_style_sender)
+            worksheet.cell(row=row+1, column=col+1).value = "s"
+            worksheet.cell(row=row+1, column=col+1).style = loc_style_sender
         else:
-            worksheet.write(row, col, "", loc_style)
+            worksheet.cell(row=row+1, column=col+1).value = ""
+            worksheet.cell(row=row+1, column=col+1).style = loc_style
         col += 1
     # loop over ECUs ends here
     return col
 
 
 def write_excel_line(worksheet, row, col, row_array, style):
-    # type: (xlsxwriter.workbook.Worksheet, int, int, typing.Sequence[typing.Any], xlsxwriter.workbook.Format) -> int
+    # type: (openpyxl.workbook.Worksheet, int, int, typing.Sequence[typing.Any], xlsxwriter.workbook.Format) -> int
     for item in row_array:
-        worksheet.write(row, col, item, style)
+        worksheet.cell(row=row+1, column=col+1).value = item
+        if style != 0:
+            worksheet.cell(row=row + 1, column=col + 1).style = style
         col += 1
     return col
 
@@ -107,7 +114,6 @@ def dump(db, filename, **options):
     additional_signal_columns = [x for x in options.get("additionalSignalAttributes", "").split(",") if x]
     additional_frame_columns = [x for x in options.get("additionalFrameAttributes", "").split(",") if x]
 
-
     head_top = [
         'ID',
         'Frame Name',
@@ -124,47 +130,60 @@ def dump(db, filename, **options):
         'Byteorder']
     head_tail = ['Value', 'Name / Phys. Range', 'Function / Increment Unit']
 
-    workbook = xlsxwriter.Workbook(filename)
+    workbook = openpyxl.Workbook()
     # ws_name = os.path.basename(filename).replace('.xlsx', '')
     # worksheet = workbook.add_worksheet('K-Matrix ' + ws_name[0:22])
-    worksheet = workbook.add_worksheet('K-Matrix ')
+    worksheet = workbook.active
+    worksheet.title = 'K-Matrix '
+    worksheet.sheet_properties.outlinePr.summaryBelow = False
+
     global sty_header
-    sty_header = workbook.add_format({'bold': True,
-                                      'rotation': 90,
-                                      'font_name': 'Verdana',
-                                      'font_size': 8,
-                                      'align': 'center',
-                                      'valign': 'center'})
-    global sty_first_frame
-    sty_first_frame = workbook.add_format({'font_name': 'Verdana',
-                                           'font_size': 8,
-                                           'font_color': 'black', 'top': 1})
     global sty_white
-    sty_white = workbook.add_format({'font_name': 'Verdana',
-                                     'font_size': 8,
-                                     'font_color': 'white'})
+    global sty_first_frame
     global sty_norm
-    sty_norm = workbook.add_format({'font_name': 'Verdana',
-                                    'font_size': 8,
-                                    'font_color': 'black'})
+    global sty_green
+    global sty_green_first_frame
+    global sty_sender
+    global sty_sender_first_frame
+    global sty_sender_green
+    global sty_sender_green_first_frame
+
+    sty_header = NamedStyle(name="sty_header")
+    sty_header.font = Font(bold=True, size=8, name='Verdana')
+    sty_header.alignment = Alignment(text_rotation=90, vertical='center', horizontal='center')
+
+    sty_first_frame = NamedStyle(name="sty_first_frame")
+    sty_first_frame.font = Font(bold=True, size=8, name='Verdana', color='ff000000')
+    sty_first_frame.border = Border(top=Side(border_style='thin'))
+
+    sty_white = NamedStyle(name="sty_white")
+    sty_white.font = Font(bold=True, size=8, name='Verdana', color='00ffffff')
+
+    sty_norm = NamedStyle(name="sty_norm")
+    sty_norm.font = Font(bold=True, size=8, name='Verdana', color='ff000000')
 
     # ECUMatrix-Styles
-    global sty_green
-    sty_green = workbook.add_format({'pattern': 1, 'fg_color': '#CCFFCC'})
-    global sty_green_first_frame
-    sty_green_first_frame = workbook.add_format(
-        {'pattern': 1, 'fg_color': '#CCFFCC', 'top': 1})
-    global sty_sender
-    sty_sender = workbook.add_format({'pattern': 0x04, 'fg_color': '#C0C0C0'})
-    global sty_sender_first_frame
-    sty_sender_first_frame = workbook.add_format(
-        {'pattern': 0x04, 'fg_color': '#C0C0C0', 'top': 1})
-    global sty_sender_green
-    sty_sender_green = workbook.add_format(
-        {'pattern': 0x04, 'fg_color': '#C0C0C0', 'bg_color': '#CCFFCC'})
-    global sty_sender_green_first_frame
-    sty_sender_green_first_frame = workbook.add_format(
-        {'pattern': 0x04, 'fg_color': '#C0C0C0', 'bg_color': '#CCFFCC', 'top': 1})
+    sty_green = NamedStyle(name="sty_green")
+    sty_green.fill = PatternFill(patternType='solid', fgColor='CCFFCC')
+    # sty_green = workbook.add_format({'pattern': 1, 'fg_color': '#CCFFCC'})
+
+    sty_green_first_frame = NamedStyle(name="sty_green_first_frame")
+    sty_green_first_frame.fill = PatternFill(patternType='solid', fgColor='CCFFCC')
+    sty_green_first_frame.border = Border(top=Side(border_style='thin'))
+
+    sty_sender = NamedStyle(name="sty_sender")
+    sty_sender.fill = PatternFill(patternType='lightGrid', fgColor='C0C0C0')
+
+    sty_sender_first_frame = NamedStyle(name="sty_sender_first_frame")
+    sty_sender_first_frame.fill = PatternFill(patternType='lightGrid', fgColor='C0C0C0')
+    sty_sender_first_frame.border = Border(top=Side(border_style='thin'))
+
+    sty_sender_green = NamedStyle(name="sty_sender_green")
+    sty_sender_green.fill = PatternFill(patternType='lightGrid', fgColor='C0C0C0', bgColor='CCFFCC')
+
+    sty_sender_green_first_frame = NamedStyle(name="sty_sender_green_first_frame")
+    sty_sender_green_first_frame.fill = PatternFill(patternType='lightGrid', fgColor='C0C0C0', bgColor='CCFFCC')
+    sty_sender_green_first_frame.border = Border(top=Side(border_style='thin'))
 
     row_array = head_top
     head_start = len(row_array)
@@ -174,18 +193,19 @@ def dump(db, filename, **options):
     row_array += ecu_list
 
     for col in range(0, len(row_array)):
-        worksheet.set_column(col, col, 2)
+        worksheet.column_dimensions[openpyxl.utils.get_column_letter(col+1)] = ColumnDimension(worksheet, customWidth=True)
+        worksheet.column_dimensions[openpyxl.utils.get_column_letter(col + 1)].width = 2
 
     row_array += head_tail
 
     additional_frame_start = len(row_array)
 
     # set width of selected Cols
-    worksheet.set_column(0, 0, 3.57)
-    worksheet.set_column(1, 1, 21)
-    worksheet.set_column(3, 3, 12.29)
-    worksheet.set_column(7, 7, 21)
-    worksheet.set_column(8, 8, 30)
+    worksheet.column_dimensions[openpyxl.utils.get_column_letter(1)].width = 3.57
+    worksheet.column_dimensions[openpyxl.utils.get_column_letter(2)].width = 21
+    worksheet.column_dimensions[openpyxl.utils.get_column_letter(3)].width = 12.29
+    worksheet.column_dimensions[openpyxl.utils.get_column_letter(4)].width = 21
+    worksheet.column_dimensions[openpyxl.utils.get_column_letter(5)].width = 30
 
     for additional_col in additional_frame_columns:
         row_array.append("frame." + additional_col)
@@ -243,6 +263,7 @@ def dump(db, filename, **options):
             write_excel_line(worksheet, row, temp_col, row_array, frame_style)
             row += 1
 
+        first_fold_row = row+1
         # iterate over signals
         for sig_idx in sorted(sig_hash.keys()):
             sig = sig_hash[sig_idx]
@@ -258,8 +279,9 @@ def dump(db, filename, **options):
                 for val in sorted(sig.values.keys()):
                     row_array = canmatrix.formats.xls_common.get_frame_info(db, frame)
                     front_col = write_excel_line(worksheet, row, 0, row_array, frame_style)
-                    if frame_style != sty_first_frame:
-                        worksheet.set_row(row, None, None, {'level': 1})
+
+                    if row >= first_fold_row:    
+                        worksheet.row_dimensions[row+1].outline_level = 1
 
                     col = head_start
                     col = write_ecu_matrix(ecu_list, sig, frame, worksheet, row, col, frame_style)
@@ -272,7 +294,6 @@ def dump(db, filename, **options):
                         temp = getattr(sig, item, "")
                         back_row.append(temp)
 
-
                     write_excel_line(worksheet, row, col + 2, back_row, signal_style)
                     write_excel_line(worksheet, row, col, [val, sig.values[val]], value_style)
 
@@ -284,12 +305,13 @@ def dump(db, filename, **options):
                     frame_style = sty_white
                     value_style = sty_norm
                 # loop over values ends here
+
             # no valuetable available
             else:
                 row_array = canmatrix.formats.xls_common.get_frame_info(db, frame)
                 front_col = write_excel_line(worksheet, row, 0, row_array, frame_style)
-                if frame_style != sty_first_frame:
-                    worksheet.set_row(row, None, None, {'level': 1})
+                if row >= first_fold_row:    
+                    worksheet.row_dimensions[row+1].outline_level = 1
 
                 col = head_start
                 col = write_ecu_matrix(ecu_list, sig, frame, worksheet, row, col, frame_style)
@@ -318,96 +340,22 @@ def dump(db, filename, **options):
         # loop over signals ends here
     # loop over frames ends here
 
-    worksheet.autofilter(0, 0, row, len(head_top) +
-                         len(head_tail) + len(db.ecus))
-    worksheet.freeze_panes(1, 0)
+    worksheet.auto_filter.ref = worksheet.dimensions
+    worksheet.freeze_panes = worksheet['B2']
     # save file
-    workbook.close()
+    workbook.save(filename=filename)
 
 
-def read_xlsx(file, **args):
-    # type: (typing.Any, **typing.Any) -> typing.Tuple[typing.Dict[typing.Any, str], typing.List[typing.Dict[str, str]]]
-    # from: Hooshmand zandi http://stackoverflow.com/a/16544219
-    import zipfile
-    from xml.etree.ElementTree import iterparse
-
-    sheet = args.get("sheet", 1)
-    is_header = args.get("header", False)
-
-    rows = []  # type: typing.List[typing.Dict[str, str]]
-    row = {}
-    header = {}
-    z = zipfile.ZipFile(file)
-
-    # Get shared strings
-    strings = [el.text for e, el
-               in iterparse(z.open('xl/sharedStrings.xml'))
-               if el.tag.endswith('}t')
-               ]  # type: typing.List[str]
-    value = ''
-
-    # Open specified worksheet
-    for e, el in iterparse(z.open('xl/worksheets/sheet%d.xml' % sheet)):
-        # get value or index to shared strings
-        if el.tag.endswith('}v'):                                   # <v>84</v>
-            value = el.text
-        if el.tag.endswith(
-                '}c'):                                   # <c r="A3" t="s"><v>84</v></c>
-            # If value is a shared string, use value as an index
-
-            if el.attrib.get('t') == 's':
-                value = strings[int(value)]
-
-            # split the row/col information so that the row letter(s) can be separate
-            letter = el.attrib['r']  # type: str         # AZ22
-            while letter[-1].isdigit():
-                letter = letter[:-1]
-
-            # if it is the first row, then create a header hash for the names that COULD be used
-            if not rows:
-                header[letter] = value.strip()
-            else:
-                if value != '':
-                    # if there is a header row, use the first row's names as the row hash index
-                    if is_header is True and letter in header:
-                        row[header[letter]] = value
-                    else:
-                        row[letter] = value
-
-            value = ''
-        if el.tag.endswith('}row'):
-            rows.append(row)
-            row = {}
-    z.close()
-    return header, rows
-
-
-def get_if_possible(row, value, default=None):
-    # type: (typing.Mapping[str, str], str, typing.Optional[str]) -> typing.Union[str, None]
-    if value in row:
-        return row[value].strip()
-    else:
-        return default
-
-
-def load(filename, **options):
+def load(file, **options):
     # type: (typing.BinaryIO, **str) -> canmatrix.CanMatrix
-    # use xlrd excel reader if available, because its more robust
-    if options.get('xlsxLegacy', False) is True:
-        logger.error("xlsx: using legacy xlsx-reader - please get xlrd working for better results!")
-    else:
-        import canmatrix.formats.xls as xls_loader  # we need alias, otherwise we hide the globally imported canmatrix
-        return xls_loader.load(filename, **options)
+
+    all_ecu_names = []
 
     # else use this hack to read xlsx
     motorola_bit_format = options.get("xlsMotorolaBitFormat", "msbreverse")
-
-    sheet = read_xlsx(filename, sheet=1, header=True)
+    workbook = openpyxl.open(file)
+    sheet = workbook._sheets[0]
     db = canmatrix.CanMatrix()
-    all_letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-    letter_index = list(all_letters)
-    letter_index += ["%s%s" % (a, b) for a in all_letters for b in all_letters]
-
     # Defines not imported...
     db.add_frame_defines("GenMsgDelayTime", 'INT 0 65535')
     db.add_frame_defines("GenMsgCycleTimeActive", 'INT 0 65535')
@@ -417,24 +365,20 @@ def load(filename, **options):
     db.add_signal_defines("GenSigSNA", 'STRING')
 
     ecu_start = ecu_end = 0
-    if 'Byteorder' in list(sheet[0].values()):
-        for key in sheet[0]:
-            if sheet[0][key].strip() == 'Byteorder':
-                ecu_start = letter_index.index(key) + 1
-                break
-    else:
-        for key in sheet[0]:
-            if sheet[0][key].strip() == 'Signal Not Available':
-                ecu_start = letter_index.index(key) + 1
 
-    for key in sheet[0]:
-        if sheet[0][key].strip() == 'Value':
-            ecu_end = letter_index.index(key)
+    column_heads = [sheet.cell(1,a).value for a in range(1, sheet.max_column+1)]
+
+    if 'Byteorder' in column_heads:
+        ecu_start = column_heads.index('Byteorder') + 1
+    else:
+        ecu_start = column_heads.index('Signal Not Available') + 1
+
+    ecu_end = column_heads.index('Value')
 
     # ECUs:
     for x in range(ecu_start, ecu_end):
-        db.add_ecu(canmatrix.Ecu(sheet[0][letter_index[x]]))
-
+        db.add_ecu(canmatrix.Ecu(column_heads[x]))
+        all_ecu_names.append(column_heads[x])
     # initialize:
     frame_id = None
     signal_name = ""
@@ -442,27 +386,40 @@ def load(filename, **options):
     new_frame = None  # type: typing.Optional[canmatrix.Frame]
     new_signal = None  # type: typing.Optional[canmatrix.Signal]
 
-    for row in sheet[1]:
+    def get_if_possible(my_row, my_value, default=None):
+        if my_value in column_heads and my_row[column_heads.index(my_value)].value is not None:
+            return my_row[column_heads.index(my_value)].value
+        else:
+            return default
+
+    for row in sheet.rows:
         # ignore empty row
-        if 'ID' not in row:
+        if row[column_heads.index('ID')].value is None or row[column_heads.index('ID')].value == 'ID':
             continue
-        # new frame detected
-        if row['ID'] != frame_id:
+
+            # new frame detected
+        if row[column_heads.index('ID')].value != frame_id:
             # new Frame
-            frame_id = row['ID']
-            frame_name = row['Frame Name']
+            frame_id = row[column_heads.index('ID')].value
+            frame_name = row[column_heads.index('Frame Name')].value
             cycle_time = get_if_possible(row, 'Cycle Time [ms]', '0')
             launch_type = get_if_possible(row, 'Launch Type')
             dlc = 8
+                    
             # launch_param = get_if_possible(row, 'Launch Parameter', '0')
             # launch_param = str(int(launch_param))
 
             if frame_id.endswith("xh"):
-                new_frame = canmatrix.Frame(frame_name, arbitration_id=int(frame_id[:-2], 16), size=dlc)
-                new_frame.arbitration_id.extended = True
+                new_frame = canmatrix.Frame(frame_name, canmatrix.ArbitrationId(int(frame_id[:-2], 16), extended=True), size=dlc)
             else:
                 new_frame = canmatrix.Frame(frame_name, arbitration_id=int(frame_id[:-1], 16), size=dlc)
 
+            for col_head in column_heads:
+                if col_head.startswith("frame."):
+                    command_str = col_head.replace("frame", "new_frame")
+                    command_str += "=" + str(row[column_heads.index(col_head)].value)
+                    exec(command_str)
+                    
             db.add_frame(new_frame)
 
             # eval launch_type
@@ -474,13 +431,13 @@ def load(filename, **options):
             new_frame.cycle_time = cycle_time
 
         # new signal detected
-        if 'Signal Name' in row and row['Signal Name'] != signal_name:
+        if get_if_possible(row, 'Signal Name') != signal_name:
             receiver = []  # type: typing.List[str]
-            start_byte = int(row["Signal Byte No."])
-            start_bit = int(row['Signal Bit No.'])
-            signal_name = row['Signal Name']
+            start_byte = int(get_if_possible(row, 'Signal Byte No.', "0"))
+            start_bit = int(get_if_possible(row, 'Signal Bit No.', "0"))
+            signal_name = get_if_possible(row, 'Signal Name')
             signal_comment = get_if_possible(row, 'Signal Function')
-            signal_length = int(row['Signal Length [Bit]'])
+            signal_length = int(get_if_possible(row, 'Signal Length [Bit]', 0))
             # signal_default = get_if_possible(row, 'Signal Default')
             # signal_sna = get_if_possible(row, 'Signal Not Available')
             multiplex = None  # type: typing.Union[str, int, None]
@@ -503,8 +460,7 @@ def load(filename, **options):
             is_signed = False
 
             if signal_name != "-":
-                for x in range(ecu_start, ecu_end):
-                    ecu_name = sheet[0][letter_index[x]].strip()
+                for ecu_name in all_ecu_names:
                     ecu_sender_receiver = get_if_possible(row, ecu_name)
                     if ecu_sender_receiver is not None:
                         if 's' in ecu_sender_receiver:
@@ -531,10 +487,14 @@ def load(filename, **options):
                             bitNumbering=1,
                             startLittle=True
                         )
-                new_frame.add_signal(new_signal)
-                new_signal.add_comment(signal_comment)
+                                    
+                if signal_name is not None:
+                    new_frame.add_signal(new_signal)
+                    new_signal.add_comment(signal_comment)
                 # function = get_if_possible(row, 'Function / Increment Unit')
         value = get_if_possible(row, 'Value')
+        if value is not None:
+            value = str(value)
         value_name = get_if_possible(row, 'Name / Phys. Range')
 
         if value_name == 0 or value_name is None:
@@ -580,6 +540,14 @@ def load(filename, **options):
             new_signal.min = None
             new_signal.max = None
 
+        for col_head in column_heads: # todo explain this possibly dangerous code with eval
+            if col_head.startswith("signal."):
+                command_str = col_head.replace("signal", "new_signal")
+                command_str += "=" + str(row[column_heads.index(col_head)].value)
+                exec(command_str)
+
+
+
     # dlc-estimation / dlc is not in xls, thus calculate a minimum-dlc:
     for frame in db.frames:
         frame.update_receiver()
